#!/usr/local/bin/perl -w

# Simple CGI Image Viewer
# Version 1.3.1 (2005-02-17)
#
# Copyright (C) 2005, 2008 Jochen Peters
#
# dev[at]emirp.net
# http://www.emirp.net/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

use strict;
use CGI;
use CGI::Carp qw(fatalsToBrowser);

# Parameters
my $imageTemplFile = 'imgviewImageTempl.html';
my $indexTemplFile = 'imgviewIndexTempl.html';
my $indexFile = 'imgview.index';
my $picDir = 'pictures/';
my $validLineRegEx = '^.+::.*::.*::.*::[0-9]+::[0-9]+';

# Global variable declarations
my $cgi;
my $imgNum;
my %imgData;
my $maxImgNum = 999;
my $indexLine = '';

# Read image number argument
$cgi = new CGI;
if($cgi->param('img')) { # If there is an img-argument, try to display corresp. image.

	$imgNum = $cgi->param('img');

	# If there is a non-digit in the img-argument, show error message and exit.
	if($imgNum =~ /\D/) {
		showErrorMessage("Invalid image number.");
		exit(1);
	}

	# Otherwise get the to $imgNum corresp. line from the index file
	$indexLine = getIndexLine();

	# If a (valid) line was found, show the image.
	if($indexLine =~ /$validLineRegEx/) {
		processLine($indexLine); # Store line data in global variable %imgData
		showImage();
	}
	else {
		showErrorMessage("Image not found.");
		exit(2);
	}
	
}
else { # If there is NO img-argument, display image index
	showIndex();
}

exit(0);


# Subroutines
sub getIndexLine {

	my $tmpLine = '';
	my $line = '';
	my $i;
	
	# Open index file and get the according line
	open(INDEX,"<$indexFile");
	$i = 0;
	while($tmpLine = <INDEX>) {
		if($tmpLine =~ /$validLineRegEx/) {
			$i++;
			if($i == $imgNum) { $line = $tmpLine; }
		}
	}
	close(INDEX);

	# Set $maxImgNum
	$maxImgNum = $i;
	
	return $line;
}

sub processLine {

	my $procLine = shift;

	chomp($procLine); # Removes the endline character
	my @splitLine = split(/::/, $procLine);

	$imgData{filename} = $splitLine[0];
	$imgData{title} = $splitLine[1];
	$imgData{description} = $splitLine[2];
	$imgData{date} = $splitLine[3];
	$imgData{width} = $splitLine[4];
	$imgData{height} = $splitLine[5];
}


sub replaceTags {

	my $text = shift;

	my $firstNum;
	my $lastNum;
	my $prevNum;
	my $nextNum;

	# Set navigation data (numbers)
	$firstNum = 1;
	$lastNum = $maxImgNum;
	$prevNum = $imgNum - 1;
	$nextNum = $imgNum + 1;
	if($prevNum < $firstNum) { $prevNum = $firstNum; }
	if($nextNum > $lastNum) { $nextNum = $lastNum; }
	
	$text =~ s/CGIIV_REPLACE_IMGNUM/$imgNum/g;
	$text =~ s/CGIIV_REPLACE_FIRSTNUM/$firstNum/g;
	$text =~ s/CGIIV_REPLACE_LASTNUM/$lastNum/g;
	$text =~ s/CGIIV_REPLACE_PREVNUM/$prevNum/g;
	$text =~ s/CGIIV_REPLACE_NEXTNUM/$nextNum/g;
	$text =~ s/CGIIV_REPLACE_IMGSRC/$picDir$imgData{filename}/g;
	$text =~ s/CGIIV_REPLACE_TITLE/$imgData{title}/g;
	$text =~ s/CGIIV_REPLACE_DESCR/$imgData{description}/g;
	$text =~ s/CGIIV_REPLACE_DATE/$imgData{date}/g;
	$text =~ s/CGIIV_REPLACE_WIDTH/$imgData{width}/g;
	$text =~ s/CGIIV_REPLACE_HEIGHT/$imgData{height}/g;

	return $text;
}


sub showImage {

	my $buffer;

	# HTML output via template file
	print "Content-type: text/html\n\n";
	open(TEMPLATE,"<$imageTemplFile");
	while($buffer = <TEMPLATE>) {
		$buffer = replaceTags($buffer);
		print $buffer;
	}
	close(TEMPLATE);
}


sub showIndex {

	my $repeatString;
	my $buffer;

	# HTML output via template file
	print "Content-type: text/html\n\n";

	open(TEMPLATE,"<$indexTemplFile");
	while($buffer = <TEMPLATE>) {
		# When REPEAT_BEGIN tags appears, stop unfiltered output and save all lines in $repeatString until REPEAT_END tag appears
		if($buffer =~ /CGIIV_REPEATED_REPLACE_BEGIN/) {
			$repeatString = '';
			while($buffer = <TEMPLATE>) {
				if($buffer =~ /CGIIV_REPEATED_REPLACE_END/) {
					last;
				}
				$repeatString .= $buffer;
			}

			# Then open index file and print the tag-replaced version of repeat lines for every valid line in the index file
			open(INDEX,"<$indexFile");
			$imgNum = 0;
			while($buffer = <INDEX>) {
				if($buffer =~ /$validLineRegEx/) {
					$imgNum++;
					processLine($buffer); # Store line data in global variable %imgData
					$buffer = $repeatString; # Make copy of $repeatString to replace the tags
					$buffer = replaceTags($buffer);
					print $buffer;
				}
			}
			close(INDEX);
		}
		# While not inside the REPEAT_BEGIN - REPEAT_END tags, just output the template file
		else {
			print $buffer;
		}
	}
	close(TEMPLATE);

}


sub showErrorMessage {
	
	my $msg = shift;

	print "Content-type: text/html\n\n";
	print "<html><head><title>CGI Image Viewer</title></head><body>\n";
	print "$msg\n";
	print "</body></html>\n";

}

